-- Create the "C:\DatabaseFiles" folder to hold the database files

USE
	master;
GO


EXECUTE sys.sp_configure
	@configname		= 'show advanced options' ,
	@configvalue	= 1;
GO


RECONFIGURE;
GO


EXECUTE sys.sp_configure
	@configname		= 'xp_cmdshell' ,
	@configvalue	= 1;
GO


RECONFIGURE;
GO


EXECUTE sys.xp_cmdshell
	N'md C:\DatabaseFiles' ,
	no_output;
GO


EXECUTE sys.sp_configure
	@configname		= 'xp_cmdshell' ,
	@configvalue	= 0;
GO


RECONFIGURE;
GO


EXECUTE sys.sp_configure
	@configname		= 'show advanced options' ,
	@configvalue	= 0;
GO


RECONFIGURE;
GO


-- Create the "TempTablesDemoDB" database

IF
	DB_ID (N'TempTablesDemoDB') IS NOT NULL
BEGIN

	ALTER DATABASE
		TempTablesDemoDB
	SET
		SINGLE_USER
	WITH
		ROLLBACK IMMEDIATE;

	DROP DATABASE
		TempTablesDemoDB;

END;
GO


CREATE DATABASE
	TempTablesDemoDB
ON PRIMARY
(
	NAME		= N'TempTablesDemoDB_Data' ,
	FILENAME	= N'C:\DatabaseFiles\TempTablesDemoDB_Data.mdf' ,
	SIZE		= 100MB ,
	FILEGROWTH	= 10%
)
LOG ON
(
	NAME		= N'TempTablesDemoDB_Log' ,
	FILENAME	= N'C:\DatabaseFiles\TempTablesDemoDB_Log.ldf' ,
	SIZE		= 10MB ,
	FILEGROWTH	= 10%
);
GO


ALTER DATABASE
	TempTablesDemoDB
SET RECOVERY
	SIMPLE;
GO


-- Create schemas

USE
	TempTablesDemoDB;
GO


CREATE SCHEMA
	Marketing;
GO


CREATE SCHEMA
	Sales;
GO


-- Create tables

CREATE TABLE
	Marketing.Customers
(
	Id					INT				NOT NULL	IDENTITY(1,1) ,
	Name				NVARCHAR(50)	NOT NULL ,
	Phone				NVARCHAR(20)	NULL ,
	BirthDate			DATE			NULL ,
	SourceURL			NVARCHAR(1000)	NULL ,
	Country				NCHAR(2)		NOT NULL ,
	LastPurchaseDate	DATE			NULL ,

	CONSTRAINT
		pk_Customers_c_Id
	PRIMARY KEY CLUSTERED
		(Id ASC)
	ON
		[PRIMARY]
)
ON
	[PRIMARY];
GO


CREATE TABLE
	Sales.Orders
(
	Id				INT				NOT NULL	IDENTITY(1,1) ,
	CustomerId		INT				NOT NULL ,
	DateAndTime		DATETIME2(0)	NOT NULL ,
	OrderStatusId	TINYINT			NOT NULL ,
	Comments		NVARCHAR(1000)	NULL ,

	CONSTRAINT
		pk_Orders_c_Id
	PRIMARY KEY CLUSTERED
		(Id ASC)
	ON
		[PRIMARY] ,

	CONSTRAINT
		fk_Orders_CustomerId_Customers_Id
	FOREIGN KEY
		(CustomerId)
	REFERENCES
		Marketing.Customers (Id)
)
ON
	[PRIMARY];
GO


-- Populate the tables

INSERT INTO
	Marketing.Customers WITH (TABLOCK)
(
	Name ,
	Phone ,
	BirthDate ,
	SourceURL ,
	Country ,
	LastPurchaseDate
)
SELECT TOP (100000)
	Name				= N'Customer #' + CAST (ROW_NUMBER () OVER (ORDER BY (SELECT NULL) ASC) AS NVARCHAR(50)) ,
	Phone				=	CASE
								WHEN ABS (CHECKSUM (NEWID ())) % 100 <= 70
									THEN LEFT (CAST (NEWID () AS NVARCHAR(MAX)) , 20)
								ELSE
									NULL
							END ,
	BirthDate			=	CASE
								WHEN ABS (CHECKSUM (NEWID ())) % 100 <= 70
									THEN DATEADD (DAY , - ABS (CHECKSUM (NEWID ())) % (365 * 60) - (365 * 20) , SYSDATETIME ())
								ELSE
									NULL
							END ,
	SourceURL			=	CASE
								WHEN ABS (CHECKSUM (NEWID ())) % 100 <= 80
									THEN N'http://www.abc.com/' + CAST (NEWID () AS NVARCHAR(1000))
								ELSE
									NULL
							END ,
	Country				=	CASE
								WHEN
									RandomNumbers.RandomNumber BETWEEN 1 AND 40
								THEN
									N'US'
								WHEN
									RandomNumbers.RandomNumber BETWEEN 41 AND 70
								THEN
									N'CN'
								WHEN
									RandomNumbers.RandomNumber BETWEEN 71 AND 99
								THEN
									N'UK'
								WHEN
									RandomNumbers.RandomNumber = 100
								THEN
									Countries.Country
							END ,
	LastPurchaseDate	= DATEADD (DAY , - (ABS (CHECKSUM (NEWID ())) % (365)) , SYSDATETIME ())
FROM
	sys.all_columns
CROSS JOIN
	(
		VALUES
			(N'AF') ,
			(N'BE') ,
			(N'CL') ,
			(N'DK') ,
			(N'EG') ,
			(N'FR') ,
			(N'GR') ,
			(N'IL') ,
			(N'JP') ,
			(N'MT') ,
			(N'NO') ,
			(N'PT') ,
			(N'SE') ,
			(N'TR') ,
			(N'VE')
	)
	AS
		Countries (Country)
CROSS JOIN
	(
		SELECT
			RandomNumber = ABS (CHECKSUM (NEWID ())) % 100 + 1
	)
	AS
		RandomNumbers
ORDER BY
	NEWID () ASC;
GO


WITH
	Numbers
(
	Number
)
AS
(
	SELECT
		Number
	FROM
		(
			VALUES (1) , (2) , (3) , (4) , (5) , (6) , (7) , (8) , (9) , (10)
		)
		AS
			Numbers (Number)
)
INSERT INTO
	Sales.Orders WITH (TABLOCK)
(
	CustomerId ,
	DateAndTime ,
	OrderStatusId ,
	Comments
)
SELECT
	CustomerId		= ABS (CHECKSUM (NEWID ())) % 100000 + 1 ,
	DateAndTime		= DATEADD (MINUTE , - ABS (CHECKSUM (NEWID ())) % (60 * 24 * 365 * 5) , SYSDATETIME ()) ,
	OrderStatusId	= ABS (CHECKSUM (NEWID ())) % 8 + 1 ,
	Comments		=	CASE
							WHEN ABS (CHECKSUM (NEWID ())) % 100 <= 40
								THEN REPLICATE (N'Bla ' , ABS (CHECKSUM (NEWID ())) % 50)
							ELSE
								NULL
						END
FROM
	Marketing.Customers AS Customers
CROSS JOIN
	Numbers;
GO


-- Disable table variable deferred compilation

ALTER DATABASE SCOPED CONFIGURATION
SET DEFERRED_COMPILATION_TV = OFF;
GO
